#sessions {
    height: 40%;
    min-height: 8;
    scrollbar-size: 0 1;
    overflow-x: auto;
}

#preview_scroll {
    height: 1fr;
    scrollbar-size: 0 1;
}

#stats_chart {
    height: 1fr;
}

#stats_loading {
    height: 1fr;
    content-align: center middle;
}
